from lantz import Feat, DictFeat, Action
from lantz.errors import InstrumentError
from lantz.messagebased import MessageBasedDriver

from pint import UnitRegistry

from time import sleep

class CLD101XLP(MessageBasedDriver):


    DEFAULTS = {
        'COMMON': {
            'write_termination': '\n',
            'read_termination': '\n'
        }
    }

    COM_DELAY = 0.2

    ureg = UnitRegistry()

    def write(self, *args, **kwargs):
        super().write(*args, **kwargs)
        sleep(self.COM_DELAY)
        return

    @Feat(read_once=True)
    def idn(self):
        return self.query('*IDN?')

    @Feat()
    def key_locked(self):
        return bool(int(self.query('OUTP:PROT:KEYL:TRIP?')))

    @Feat(values={'C': 'CEL', 'F': 'FAR', 'K': 'K'})
    def temperature_unit(self):
        self.t_unit = self.query('UNIT:TEMP?')
        return self.t_unit

    @temperature_unit.setter
    def temperature_unit(self, value):
        self.write('UNIT:TEMP {}'.format(value))

    @Feat()
    def temperature(self):
        return float(self.query('MEAS:SCAL:TEMP?'))

    @Feat()
    def temperature_setpoint(self):
        return float(self.query('SOUR2:TEMP?'))

    @Action()
    def read_error_queue(self):
        no_error = "+0,'No error'"
        error = inst.query('SYST:ERR:NEXT?')
        while(error != no_error):
            print(error)
            error = inst.query('SYST:ERR:NEXT?')


    @Feat(values={False: '0', True: '1'})
    def ld_state(self):
        return self.query('OUTP1:STAT?')

    @ld_state.setter
    def ld_state(self, value):
        self.write('OUTP1:STAT {}'.format(value))

    @Feat(units='A', limits=(0,0.9))
    def ld_current_setpoint(self):
        return float(self.query('SOUR:CURR?'))

    @Feat(units='A', limits=(0.0, 0.9))
    def ld_current(self):
        return float(self.query('MEAS:CURR?'))

    @ld_current.setter
    def ld_current(self, value):
        inst.write('SOUR:CURR {:.5f}'.format(value))

    @DictFeat(units='W', keys={'photodiode', 'pd', 'thermopile', 'tp', 'power meter'})
    def ld_power(self, method):
        query = 'MEAS:POW{}?'
        ml = method.lower()
        if ml in {'photodiode', 'pd'}:
            method_val = 2
        elif ml in {'thermopile', 'tp', 'power meter'}:
            method_val = 3
        return float(self.query(query.format(method_val)))

    @Feat(values={False: '0', True: '1'})
    def tec_state(self):
        return self.query('OUTP2:STAT?')

    @tec_state.setter
    def tec_state(self, value):
        self.write('OUTP2:STAT {}'.format(value))


    @Action()
    def turn_on_seq(self, temp_error=0.05, current_error=0.005*ureg.milliamp):
        if self.ld_state == 1:
            print("Laser is already ON!")
            return

        #Turn ON sequence:
        #   1. TEC ON
        #   2. Wait for temperature == set_temperature
        #   3. LD ON
        #   4. Wait for current == set_current

        # 1. TEC ON
        self.tec_state = 1

        # 2. Wait
        setpoint = self.temperature_setpoint
        while(abs(setpoint-self.temperature)>temp_error):pass


        # 3. LD ON
        self.ld_state = 1

        # 4. Wait
        setpoint = self.ld_current_setpoint
        while(abs(setpoint.m-self.ld_current.m)>current_error.m):pass

    @Action()
    def turn_off_seq(self, current_error=0.005*ureg.milliamp):
        #Turn OFF sequence:
        #   1. LD OFF
        #   2. Wait for current == 0
        #   3. TEC OFF

        # 1. LD OFF
        self.ld_state = 0

        # 2. Wait
        while(abs(self.ld_current.m) > current_error.m):pass

        # 1. TEC OFF
        self.tec_state = 0



if __name__ == '__main__':
    import logging
    import sys
    from lantz.log import log_to_screen
    log_to_screen(logging.CRITICAL)
    #res_name = sys.argv[1]

    res_names = ['USB0::0x1313::0x804F::SERIALNUMBER::INSTR', 'USB0::0x1313::0x804F::SERIALNUMBER::INSTR']
    print('update res_names!')

    fmt_str = "{:<30}|{:>30}"
    on_time = 20

    for resource in res_names:

        with CLD101XLP(resource) as inst:

    # with CLD101XLP(res_name) as inst:
            print(fmt_str.format("Temperature unit", inst.temperature_unit))
            print(fmt_str.format("Device name", inst.query('*IDN?')))
            print(fmt_str.format("LD state", inst.ld_state))
            print(fmt_str.format("TEC state", inst.tec_state))
            print(fmt_str.format("Temp setpoint", inst.temperature_setpoint))
    #     inst.ld_current = .0885
            print(fmt_str.format("LD current", inst.ld_current))
            print(fmt_str.format("LD current setpoint", inst.ld_current_setpoint))

            print(fmt_str.format("LD state", inst.ld_state))
            print(fmt_str.format("TEC state", inst.tec_state))
            print(fmt_str.format("LD temperature", inst.temperature))
    #
    #     print("Turning on TEC and LD...")
    #     inst.turn_on_seq()
    #     #print(fmt_str.format("LD power (via photodiode)", inst.ld_power['pd']))
    #     #print(fmt_str.format("LD power (via thermopile)", inst.ld_power['tp']))
            print(fmt_str.format("LD state", inst.ld_state))
            print(fmt_str.format("TEC state", inst.tec_state))
            print(fmt_str.format("LD temperature", inst.temperature))
    #     print(fmt_str.format("LD current", inst.ld_current))
    #     print(fmt_str.format("LD current setpoint", inst.ld_current_setpoint))
    #
    #
    #     inst.ld_current = .025
    #     print(fmt_str.format("LD current", inst.ld_current))
    #     print(fmt_str.format("LD current setpoint", inst.ld_current_setpoint))
    #     #print(fmt_str.format("LD power (via photodiode)", inst.ld_power['pd']))
    #     #print(fmt_str.format("LD power (via thermopile)", inst.ld_power['tp']))
    #     sleep(on_time)
    #
    #     print("Turning off TEC and LD...")
    #     inst.turn_off_seq()
    #     print(fmt_str.format("LD state", inst.ld_state))
    #     print(fmt_str.format("TEC state", inst.tec_state))
    #     print(fmt_str.format("LD current", inst.ld_current))
    #     print(fmt_str.format("LD current setpoint", inst.ld_current_setpoint))



# -*- coding: utf-8 -*-
"""
    lantz.drivers.toptica.dlcpro.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Implementation of Toptica DLC-pro laser controller

    Author: Alexandre Bourassa
    Date: 21/04/2017
"""

import numpy as np
from lantz import Action, Feat, DictFeat, ureg
from lantz.messagebased import MessageBasedDriver

class DLC(MessageBasedDriver):

        DEFAULTS = {
            'COMMON': {
                'write_termination': '\n',
                'read_termination': '\n',
            }
        }

        def initialize(self):
            super().initialize()
            # clear welcome message
            for _ in range(5):
                self.resource.read_raw()
            return

        def set_param(self, param_name, value):
            cmd = "(param-set! '{} {})".format(param_name, value)
            self.write(cmd)
            retval = self.read()
            retval = retval.lstrip('> ')
            if retval.startswith('Error: '):
                retval = retval.lstrip('Error: ')
                retparts = retval.split()
                errcode, errmsg = int(retparts[0]), ' '.join(retparts[1:])
                errcode = int(self.read())
                raise DLCException("Error {}: {}".format(errcode, errmsg))
            return retval

        def get_param(self, param_name):
            cmd = "(param-ref '{})".format(param_name)
            self.write(cmd)
            retval = self.read()
            retval = retval.lstrip('> ')
            if retval.startswith('Error: '):
                retval = retval.lstrip('Error: ')
                retparts = retval.split()
                errcode, errmsg = int(retparts[0]), ' '.join(retparts[1:])
                raise DLCException("Error {}: {}".format(errcode, errmsg))
            return retval

        @Feat()
        def idn(self):
            """
            Returns DLCpro
            """
            return self.get_param("system-type")


        ##------------------------
        ##    Current Control
        ##------------------------

        @Feat(values={True: '#t', False: '#f'})
        def output(self):
            return self.get_param('laser1:dl:cc:enabled')

        @output.setter
        def output(self, val):
            return self.set_param('laser1:dl:cc:enabled', val)

        @Feat(values={True: '#t', False: '#f'})
        def feedforward_enabled(self):
            return self.get_param('laser1:dl:cc:feedforward-enabled')

        @feedforward_enabled.setter
        def feedforward_enabled(self, val):
            return self.set_param('laser1:dl:cc:feedforward-enabled', val)

        @Feat(units='mA')
        def current(self):
            return self.get_param('laser1:dl:cc:current-set')

        @current.setter
        def current(self, val):
            return self.set_param('laser1:dl:cc:current-set', val)

        @Feat(units='mA')
        def current_offset(self):
            return self.get_param('laser1:dl:cc:current-offset')

        @current_offset.setter
        def current_offset(self, val):
            return self.set_param('laser1:dl:cc:current-offset', val)

        @Feat(units='mA')
        def current_actual(self):
            return self.get_param('laser1:dl:cc:current-act')

        @Feat(units='mA/V')
        def feedforward_factor(self):
            return self.get_param('laser1:dl:cc:feedforward-factor')

        @feedforward_factor.setter
        def feedforward_factor(self, val):
            return self.set_param('laser1:dl:cc:feedforward-factor', val)

        @Feat()
        def current_external_input_signal(self):
            return int(self.get_param('laser1:dl:cc:external-input:signal'))

        @current_external_input_signal.setter
        def current_external_input_signal(self, val):
            return self.set_param('laser1:dl:cc:external-input:signal', int(val))

        @Feat()
        def current_external_input_factor(self):
            return float(self.get_param('laser1:dl:cc:external-input:factor'))

        @current_external_input_factor.setter
        def current_external_input_factor(self, val):
            return self.set_param('laser1:dl:cc:external-input:factor', float(val))

        @Feat(values={True: '#t', False: '#f'})
        def current_external_input_enabled(self):
            return self.get_param('laser1:dl:cc:external-input:enabled')

        @current_external_input_enabled.setter
        def current_external_input_enabled(self, val):
            return self.set_param('laser1:dl:cc:external-input:enabled', val)


        ##------------------------
        ##    Piezo Control
        ##------------------------
        @Feat(units='V')
        def piezo_voltage_actual(self):
            return self.get_param('laser1:dl:pc:voltage-act')

        @Feat(values={True: '#t', False: '#f'})
        def piezo_enabled(self):
            return self.get_param('laser1:dl:pc:enabled')

        @piezo_enabled.setter
        def piezo_enabled(self, val):
            return self.set_param('laser1:dl:pc:enabled', val)

        @Feat(units='V')
        def piezo_vmax(self):
            return self.get_param('laser1:dl:pc:voltage-max')

        @Feat(units='V')
        def piezo_vmin(self):
            return self.get_param('laser1:dl:pc:voltage-min')

        @Feat(units='V')
        def piezo_voltage(self):
            return self.get_param('laser1:dl:pc:voltage-set')

        @piezo_voltage.setter
        def piezo_voltage(self, val):
            return self.set_param('laser1:dl:pc:voltage-set', val)

        @Feat()
        def piezo_external_input_signal(self):
            return int(self.get_param('laser1:dl:pc:external-input:signal'))

        @piezo_external_input_signal.setter
        def piezo_external_input_signal(self, val):
            return self.set_param('laser1:dl:pc:external-input:signal', int(val))

        @Feat()
        def piezo_external_input_factor(self):
            return float(self.get_param('laser1:dl:pc:external-input:factor'))

        @piezo_external_input_factor.setter
        def piezo_external_input_factor(self, val):
            return self.set_param('laser1:dl:pc:external-input:factor', float(val))

        @Feat(values={True: '#t', False: '#f'})
        def piezo_external_input_enabled(self):
            return self.get_param('laser1:dl:pc:external-input:enabled')

        @piezo_external_input_enabled.setter
        def piezo_external_input_enabled(self, val):
            return self.set_param('laser1:dl:pc:external-input:enabled', val)

        ##------------------------
        ##    Scan Control
        ##------------------------

        @Feat(values={True: '#t', False: '#f'})
        def scan_enabled(self):
            return self.get_param('laser1:scan:enabled')

        @scan_enabled.setter
        def scan_enabled(self, val):
            return self.set_param('laser1:scan:enabled', val)

        @Feat()
        def scan_channel(self):
            return int(self.get_param('laser1:scan:output-channel'))

        @scan_channel.setter
        def scan_channel(self, val):
            return self.set_param('laser1:scan:output-channel', int(val))

        @Feat()
        def scan_amplitude(self):
            return float(self.get_param('laser1:scan:amplitude'))

        @scan_amplitude.setter
        def scan_amplitude(self, val):
            return self.set_param('laser1:scan:amplitude', float(val))

        @Feat()
        def scan_offset(self):
            return float(self.get_param('laser1:scan:offset'))

        @scan_offset.setter
        def scan_offset(self, val):
            return self.set_param('laser1:scan:offset', float(val))

        @Feat(units='Hz')
        def scan_frequency(self):
            return self.get_param('laser1:scan:frequency')

        @scan_frequency.setter
        def scan_frequency(self, val):
            return self.set_param('laser1:scan:frequency', val)

        @Feat(values={'sine': 0, 'triangle': 1})
        def scan_signal_type(self):
            return int(self.get_param('laser1:scan:signal-type'))

        @scan_signal_type.setter
        def scan_signal_type(self, val):
            return self.set_param('laser1:scan:signal-type', val)



class DLCException(Exception):
    pass



import ctypes
import time
from io import BytesIO
import struct
import numpy as np

from lantz.foreign import LibraryDriver
from lantz import Feat, DictFeat, Action

TTREADMAX = 131072
FIFOFULL = 0x0003

t2_wraparound = 210698240

class PH300(LibraryDriver):

    LIBRARY_NAME = 'phlib64.dll'
    LIBRARY_PREFIX = 'PH_'

    def __init__(self, device_idx):
        super().__init__()
        self.device_idx = device_idx
        return

    def call(self, func_name, *args):
        err = getattr(self.lib, func_name)(*args)
        if not err:
            return
        else:
            s = ctypes.create_string_buffer(50)
            self.lib.GetErrorString(s,err)
            raise Exception(s.value.decode('ascii'))

    def initialize(self):
        serial = ctypes.create_string_buffer(8)
        self.call('OpenDevice', self.device_idx, serial)
        self.call('Initialize', self.device_idx, 2)
        features = ctypes.c_int()
        self.call('GetFeatures', self.device_idx, ctypes.byref(features))
        return

    @DictFeat(keys=(0, 1))
    def count_rate(self, channel):
        rate = ctypes.c_int()
        self.call('GetCountRate', self.device_idx, channel, ctypes.byref(rate))
        return rate.value

    @Feat()
    def resolution(self):
        resolution = ctypes.c_double(0.0)
        self.call('GetResolution', self.device_idx, ctypes.byref(resolution))
        return resolution.value

    @Action()
    def set_input_cfd(self, channel, level, zero_cross):
        self.call('SetInputCFD', self.device_idx, channel, level, zero_cross)
        return

    @Action()
    def set_sync_div(self, sync_div):
        self.call('SetSyncDiv', self.device_idx, sync_div)
        return

    @Action()
    def set_binning(self, binning):
        self.call('SetBinning', self.device_idx, binning)
        return

    @Action()
    def set_offset(self, offset):
        self.call('SetOffset', self.device_idx, offset)
        return

    @Action(units=('ms'))
    def start_measurement(self, measurement_time):
        self.call('StartMeas', self.device_idx, int(measurement_time))
        return

    @Action()
    def stop_measurement(self):
        self.call('StopMeas', self.device_idx)
        return

    @Feat()
    def elapsed_measurement_time(self):
        elapsed = ctypes.c_double(0.0)
        self.call('GetElapsedMeasTime', self.device_idx, ctypes.byref(elapsed))
        return elapsed.value

    @Feat()
    def flags(self):
        flags = ctypes.c_uint()
        self.call('GetFlags', self.device_idx, ctypes.byref(flags))
        return flags.value

    @Action()
    def read_fifo(self, nvalues=TTREADMAX):
        databuf = BytesIO()
        buf = (ctypes.c_uint * nvalues)()
        n_to_read = ctypes.c_uint(nvalues)
        n_read = ctypes.c_uint(0)
        while 1:
            if self.flags & FIFOFULL:
                break
            retcode = self.lib.ReadFiFo(self.device_idx, ctypes.byref(buf), n_to_read, ctypes.byref(n_read))
            if retcode < 0:
                break
            if n_read.value:
                databuf.write(bytes(buf)[:n_read.value * 4])
            else:
                break
        databuf.seek(0)
        return databuf

    @Action()
    def read_timestamps(self, nvalues=TTREADMAX):
        databuf = self.read_fifo(nvalues=nvalues)
        c1 = list()
        c2 = list()
        overflow_time = 0
        resolution = self.resolution
        while 1:
            chunk = databuf.read(4)
            if not chunk:
                break
            chunk = struct.unpack('<I', chunk)[0]
            time = chunk & 0x0FFFFFFF
            channel = (chunk & 0xF0000000) >> 28
            if channel == 0xF:
                markers = time & 0xF
                if not markers:
                    overflow_time += t2_wraparound
                else:
                    truetime = overflow_time + time
            else:
                if channel > 4:
                    raise RuntimeError('invalid channel encountered: {}'.format(channel))
                else:
                    # truetime = (overflow_time + time) / 1e12 * resolution
                    truetime = (overflow_time + time) * resolution
                    if channel >= 1:
                        c2.append(truetime)
                    else:
                        c1.append(truetime)
        return c1, c2

    def finalize(self):
        self.call('CloseDevice', self.device_idx)
        return



# -*- coding: utf-8 -*-
"""
    lantz.drivers.legacy.ni.daqmx.channels
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Implementation of specialized channel classes.

    :copyright: 2015 by Lantz Authors, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""

from .base import Channel, Task
from .constants import Constants


class VoltageInputChannel(Channel):
    """Creates channel(s) to measure voltage and adds the channel(s)
    to the task you specify with taskHandle.

    If your measurement requires the use of internal excitation or you need
    the voltage to be scaled by excitation, call DAQmxCreateAIVoltageChanWithExcit.

    :param phys_channel: The names of the physical channels to use
                         to create virtual channels. You can specify
                         a list or range of physical channels.
    :param channel_name: The name(s) to assign to the created virtual channel(s).
                         If you do not specify a name, NI-DAQmx uses the physical
                         channel name as the virtual channel name. If you specify
                         your own names for nameToAssignToChannel, you must use the
                         names when you refer to these channels in other NI-DAQmx
                         functions.

                         If you create multiple virtual channels with one call to
                         this function, you can specify a list of names separated by
                         commas. If you provide fewer names than the number of
                         virtual channels you create, NI-DAQmx automatically assigns
                         names to the virtual channels.

    :param terminal: {'default', 'rse', 'nrse', 'diff', 'pseudodiff'}
      The input terminal configuration for the channel:

        'default'
          At run time, NI-DAQmx chooses the default terminal
          configuration for the channel.

        'rse'
          Referenced single-ended mode

        'nrse'
          Nonreferenced single-ended mode

        'diff'
          Differential mode

        'pseudodiff'
          Pseudodifferential mode

    :param min_val: The minimum value, in units, that you expect to measure.
    :param max_val: The maximum value, in units, that you expect to measure.
    :param units: units to use to return the voltage measurements
    """

    IO_TYPE = 'AI'

    CREATE_FUN = 'CreateAIVoltageChan'

    terminal_map = dict (default = Constants.Val_Cfg_Default,
                         rse = Constants.Val_RSE,
                         nrse = Constants.Val_NRSE,
                         diff = Constants.Val_Diff,
                         pseudodiff = Constants.Val_PseudoDiff)

    def __init__(self, phys_channel, name='', terminal='default',
                 min_max=(-10., 10.), units='volts', task=None):

        if not name:
            name = ''#phys_channel

        terminal_val = self.terminal_map[terminal]

        if units != 'volts':
            custom_scale_name = units
            units = Constants.Val_FromCustomScale
        else:
            custom_scale_name = None
            units =  Constants.Val_Volts

        self._create_args = (phys_channel, name, terminal_val,
                             min_max[0], min_max[1], units, custom_scale_name)

        super().__init__(task=task, name=name)


class VoltageOutputChannel(Channel):
    """Creates channel(s) to generate voltage and adds the channel(s)
    to the task you specify with taskHandle.

        See VoltageOutputChannel
    """

    CHANNEL_TYPE = 'AO'

    def __init__(self, phys_channel, channel_name='', terminal='default', min_max=(-1, -1), units='volts'):

        terminal_val = self.terminal_map[terminal]

        if units != 'volts':
            custom_scale_name = units
            units = Constants.FROM_CUSTOM_SCALE
        else:
            custom_scale_name = None
            units =  Constants.VOLTS

        err = self.lib.CreateAOVoltageChan(phys_channel, channel_name,
                                           min_max[0], min_max[1], units, custom_scale_name)

# Not implemented:
# DAQmxCreateAIAccelChan, DAQmxCreateAICurrentChan, DAQmxCreateAIFreqVoltageChan,
# DAQmxCreateAIMicrophoneChan, DAQmxCreateAIResistanceChan, DAQmxCreateAIRTDChan,
# DAQmxCreateAIStrainGageChan, DAQmxCreateAITempBuiltInSensorChan,
# DAQmxCreateAIThrmcplChan, DAQmxCreateAIThrmstrChanIex, DAQmxCreateAIThrmstrChanVex,
# DAQmxCreateAIVoltageChanWithExcit
# DAQmxCreateAIPosLVDTChan, DAQmxCreateAIPosRVDTChan/
# DAQmxCreateTEDSAI*

# Not implemented: DAQmxCreateAOCurrentChan
# DAQmxCreateDIChan, DAQmxCreateDOChan
# DAQmxCreateCI*, DAQmxCreateCO*


class DigitalInputChannel(Channel):
    """
    Creates channel(s) to measure digital signals and adds the
    channel(s) to the task you specify with taskHandle. You can
    group digital lines into one digital channel or separate them
    into multiple digital channels. If you specify one or more
    entire ports in lines by using port physical channel names,
    you cannot separate the ports into multiple channels. To
    separate ports into multiple channels, use this function
    multiple times with a different port each time.

    Parameters
    ----------

    lines : str

      The names of the digital lines used to create a virtual
      channel. You can specify a list or range of lines.

    name : str

      The name of the created virtual channel(s). If you create
      multiple virtual channels with one call to this function,
      you can specify a list of names separated by commas. If you
      do not specify a name, NI-DAQmx uses the physical channel
      name as the virtual channel name. If you specify your own
      names for name, you must use the names when you refer to
      these channels in other NI-DAQmx functions.

    group_by : {'line', 'all_lines'}

      Specifies whether to group digital lines into one or more
      virtual channels. If you specify one or more entire ports in
      lines, you must set grouping to 'for_all_lines':

        'line' - One channel for each line

        'all_lines' - One channel for all lines
    """


    def __init__(self, lines, name='', group_by='line'):

        if group_by == 'line':
            grouping_val = Constants.ChanPerLine
            self.one_channel_for_all_lines = False
        else:
            grouping_val = Constants.ChanForAllLines
            self.one_channel_for_all_lines = True

        self.lib.CreateDIChan(lines, name, grouping_val)


class DigitalOutputChannel(Channel):
    """
    Creates channel(s) to generate digital signals and adds the
    channel(s) to the task you specify with taskHandle. You can
    group digital lines into one digital channel or separate them
    into multiple digital channels. If you specify one or more
    entire ports in lines by using port physical channel names,
    you cannot separate the ports into multiple channels. To
    separate ports into multiple channels, use this function
    multiple times with a different port each time.

    See DigitalInputChannel
    """

    def __init__(self, lines, name='', group_by='line'):

        if group_by == 'line':
            grouping_val = Constants.ChanPerLine
            self.one_channel_for_all_lines = False
        else:
            grouping_val = Constants.ChanForAllLines
            self.one_channel_for_all_lines = True

        self.lib.CreateDOChan(lines, name, grouping_val)


class CountEdgesChannel(Channel):
    """
    Creates a channel to count the number of rising or falling
    edges of a digital signal and adds the channel to the task you
    specify with taskHandle. You can create only one counter input
    channel at a time with this function because a task can
    include only one counter input channel. To read from multiple
    counters simultaneously, use a separate task for each
    counter. Connect the input signal to the default input
    terminal of the counter unless you select a different input
    terminal.

    Parameters
    ----------

    counter : str

      The name of the counter to use to create virtual channels.

    name : str

      The name(s) to assign to the created virtual channel(s). If
      you do not specify a name, NI-DAQmx uses the physical
      channel name as the virtual channel name. If you specify
      your own names for nameToAssignToChannel, you must use the
      names when you refer to these channels in other NI-DAQmx
      functions.

      If you create multiple virtual channels with one call to
      this function, you can specify a list of names separated by
      commas. If you provide fewer names than the number of
      virtual channels you create, NI-DAQmx automatically assigns
      names to the virtual channels.

    edge : {'rising', 'falling'}

      Specifies on which edges of the input signal to increment or
      decrement the count, rising or falling edge(s).

    init : int

      The value from which to start counting.

    direction : {'up', 'down', 'ext'}

      Specifies whether to increment or decrement the
      counter on each edge:

        'up' - Increment the count register on each edge.

        'down' - Decrement the count register on each edge.

        'ext' - The state of a digital line controls the count
        direction. Each counter has a default count direction
        terminal.
    """

    CHANNEL_TYPE = 'CI'


    def __init__ (self, counter, name="", edge='rising', init=0, direction='up'):

        if edge == 'rising':
            edge_val = Constants.RISING
        else:
            edge_val = Constants.FALLING

        if direction == 'up':
            direction_val = Constants.COUNT_UP
        else:
            direction_val = Constants.COUNT_DOWN

        self.lib.CreateCICountEdgesChan(counter, name, edge_val, direction_val)


class LinearEncoderChannel(Channel):
    """
    Creates a channel that uses a linear encoder to measure linear position.
    You can create only one counter input channel at a time with this function
    because a task can include only one counter input channel. To read from
    multiple counters simultaneously, use a separate task for each counter.
    Connect the input signals to the default input terminals of the counter
    unless you select different input terminals.

    Parameters
    ----------

    counter : str

      The name of the counter to use to create virtual channels.

    name : str

      The name(s) to assign to the created virtual channel(s). If
      you do not specify a name, NI-DAQmx uses the physical
      channel name as the virtual channel name. If you specify
      your own names for nameToAssignToChannel, you must use the
      names when you refer to these channels in other NI-DAQmx
      functions.

      If you create multiple virtual channels with one call to
      this function, you can specify a list of names separated by
      commas. If you provide fewer names than the number of
      virtual channels you create, NI-DAQmx automatically assigns
      names to the virtual channels.

    decodingType : {'X1', 'X2', 'X4', 'TwoPulseCounting'}

      Specifies how to count and interpret the pulses that the encoder
      generates on signal A and signal B. X1, X2, and X4 are valid for
      quadrature encoders only. TwoPulseCounting is valid only for
      two-pulse encoders.

      X2 and X4 decoding are more sensitive to smaller changes in position
      than X1 encoding, with X4 being the most sensitive. However, more
      sensitive decoding is more likely to produce erroneous measurements
      if there is vibration in the encoder or other noise in the signals.

    ZidxEnable : bool

      Specifies whether to enable z indexing for the measurement.

    ZidxVal : float

      The value, in units, to which to reset the measurement when signal Z
      is high and signal A and signal B are at the states you specify with
      ZidxPhase.

    ZidxPhase : {'AHighBHigh', 'AHighBLow', 'ALowBHigh', 'ALowBLow'}

      The states at which signal A and signal B must be while signal Z is high
      for NI-DAQmx to reset the measurement. If signal Z is never high while
      the signal A and signal B are high, for example, you must choose a phase
      other than Constants.Val_AHighBHigh.

      When signal Z goes high and how long it stays high varies from encoder to
      encoder. Refer to the documentation for the encoder to determine the
      timing of signal Z with respect to signal A and signal B.

    units  : {'Meters', 'Inches', 'Ticks', 'FromCustomScale'}

      The units to use to return linear position measurements from the channel.

    distPerPulse : float

      The distance measured for each pulse the encoder generates. Specify this
      value in units.

    init : float

      The position of the encoder when the measurement begins. This value is
      in units.

    customScaleName : str

      The name of a custom scale to apply to the channel. To use this parameter,
      you must set units to Constants.Val_FromCustomScale. If you do not set units
      to FromCustomScale, you must set customScaleName to NULL.
    """

    def __init__(
            self,
            counter,
            name="",
            decodingType='X1',
            ZidxEnable=False,
            ZidxVal=0.0,
            ZidxPhase='AHighBHigh',
            units='Ticks',
            distPerPulse=1.0,
            init=0.0,
            customScaleName=None
    ):
        counter = str(counter)
        name = str(name)

        decodingType_map = dict(X1=Constants.Val_X1, X2=Constants.Val_X2, X4=Constants.Val_X4,
                                TwoPulseCounting=Constants.Val_TwoPulseCounting)
        ZidxPhase_map = dict(AHighBHigh=Constants.Val_AHighBHigh, AHighBLow=Constants.Val_AHighBLow,
                             ALowBHigh=Constants.Val_ALowBHigh, ALowBLow=Constants.Val_ALowBLow)
        units_map = dict(Meters=Constants.Val_Meters, Inches=Constants.Val_Inches,
                         Ticks=Constants.Val_Ticks, FromCustomScale=Constants.Val_FromCustomScale)

        decodingType_val = self._get_map_value ('decodingType', decodingType_map, decodingType)
        ZidxPhase_val = self._get_map_value ('ZidxPhase', ZidxPhase_map, ZidxPhase)
        units_val = self._get_map_value ('units', units_map, units)

        if units_val != Constants.Val_FromCustomScale:
            customScaleName = None

        CALL(
            'CreateCILinEncoderChan',
            self,
            counter,
            name,
            decodingType_val,
            bool32(ZidxEnable),
            float64(ZidxVal),
            ZidxPhase_val,
            units_val,
            float64(distPerPulse),
            float64(init),
            customScaleName
        )==0


class MeasureFrequencyChannel(Channel):
    """
    Creates a channel to measure the frequency of a digital signal
    and adds the channel to the task. You can create only one
    counter input channel at a time with this function because a
    task can include only one counter input channel. To read from
    multiple counters simultaneously, use a separate task for each
    counter. Connect the input signal to the default input
    terminal of the counter unless you select a different input
    terminal.

    Parameters
    ----------

    counter : str
      The name of the counter to use to create virtual channels.

    name : str
      The name(s) to assign to the created virtual channel(s). If
      you do not specify a name, NI-DAQmx uses the physical
      channel name as the virtual channel name. If you specify
      your own names for nameToAssignToChannel, you must use the
      names when you refer to these channels in other NI-DAQmx
      functions.

      If you create multiple virtual channels with one call to
      this function, you can specify a list of names separated by
      commas. If you provide fewer names than the number of
      virtual channels you create, NI-DAQmx automatically assigns
      names to the virtual channels.

    min_val : float
      The minimum value, in units, that you expect to measure.

    max_val : float
      The maximum value, in units, that you expect to measure.

    units : {'hertz', 'ticks', 'custom'}
      Units to use to return the measurement and to specify the
      min/max expected value.

      'hertz' - Hertz, cycles per second
      'ticks' - timebase ticks
      'custom' - use custom_scale_name to specify units

    edge : {'rising', 'falling'}
      Specifies which edges to measure the frequency or period of the signal.

    method : {'low_freq', 'high_freq', 'large_range'}
      The method used to calculate the period or frequency of the
      signal.  See the M series DAQ User Manual (371022K-01), page
      7-9 for more information.

        'low_freq'
          Use one counter that uses a constant timebase to measure
          the input signal.

        'high_freq'
          Use two counters, one of which counts pulses of the
          signal to measure during the specified measurement time.

        'large_range'
          Use one counter to divide the frequency of the input
          signal to create a lower frequency signal that the
          second counter can more easily measure.

    meas_time : float
      The length of time to measure the frequency or period of the
      signal, when meas_method is 'high_freq'.  Measurement accuracy
      increases with increased meas_time and with increased signal
      frequency.  Ensure that the meas_time is low enough to prevent
      the counter register from overflowing.

    divisor : int
      The value by which to divide the input signal, when
      meas_method is 'large_range'. The larger this value, the more
      accurate the measurement, but too large a value can cause the
      count register to roll over, resulting in an incorrect
      measurement.

    custom_scale_name : str
      The name of a custom scale to apply to the channel. To use
      this parameter, you must set units to 'custom'.  If you do
      not set units to 'custom', you must set custom_scale_name to
      None.
    """

    def __init__(self, counter, name='', min_val=1e2, max_val=1e3,
                            units="hertz", edge="rising", method="low_freq",
                            meas_time=1.0, divisor=1, custom_scale_name=None):

        self.data_type = float

        assert divisor > 0

        if method == 'low_freq':
            meas_meth_val = Constants.LOW_FREQ1_CTR
        elif method == 'high_freq':
            meas_meth_val = Constants.HIGH_FREQ2_CTR
        elif method == 'large_range':
            meas_meth_val = Constants.LARGE_RANGE2_CTR


        if units != ('hertz', 'ticks'):
            custom_scale_name = units
            units = Constants.FROM_CUSTOM_SCALE
        else:
            custom_scale_name = None
            if units == 'hertz':
                units =  Constants.HZ
            else:
                units = Contstants.TICKS

        self.lib.CreateCIFreqChan(counter, name, min_max[0], min_max[1],
                                  units_val, edge_val, meas_meth_val,
                                  meas_time, divisor, custom_scale_name)



def create_channel_frequency(self, counter, name="", units='hertz', idle_state='low',
                             delay=0.0, freq=1.0, duty_cycle=0.5):
    """
    Creates channel(s) to generate digital pulses that freq and
    duty_cycle define and adds the channel to the task.  The
    pulses appear on the default output terminal of the counter
    unless you select a different output terminal.

    Parameters
    ----------

    counter : str

      The name of the counter to use to create virtual
      channels. You can specify a list or range of physical
      channels.

    name : str

      The name(s) to assign to the created virtual channel(s). If
      you do not specify a name, NI-DAQmx uses the physical
      channel name as the virtual channel name. If you specify
      your own names for nameToAssignToChannel, you must use the
      names when you refer to these channels in other NI-DAQmx
      functions.

      If you create multiple virtual channels with one call to
      this function, you can specify a list of names separated by
      commas. If you provide fewer names than the number of
      virtual channels you create, NI-DAQmx automatically assigns
      names to the virtual channels.

    units : {'hertz'}

      The units in which to specify freq:

        'hertz' - hertz

    idle_state : {'low', 'high'}

      The resting state of the output terminal.

    delay : float

      The amount of time in seconds to wait before generating the
      first pulse.

    freq : float

      The frequency at which to generate pulses.

    duty_cycle : float

      The width of the pulse divided by the pulse period. NI-DAQmx
      uses this ratio, combined with frequency, to determine pulse
      width and the interval between pulses.

    Returns
    -------

      success_status : bool
    """
    counter = str(counter)
    name = str(name)
    units_map = dict (hertz = Constants.Val_Hz)
    idle_state_map = dict (low=Constants.Val_Low, high=Constants.Val_High)
    units_val = self._get_map_value('units', units_map, units)
    idle_state_val = self._get_map_value('idle_state', idle_state_map, idle_state)
    self.lib.CreateCOPulseChanFreq(counter, name, units_val, idle_state_val,
                                   delay, freq, (duty_cycle))

def create_channel_ticks(self, counter, name="", source="", idle_state='low',
                         delay = 0, low_ticks=1, high_ticks=1):
    """
    Creates channel(s) to generate digital pulses defined by the
    number of timebase ticks that the pulse is at a high state and
    the number of timebase ticks that the pulse is at a low state
    and also adds the channel to the task. The pulses appear on
    the default output terminal of the counter unless you select a
    different output terminal.

    Parameters
    ----------

    counter : str

      The name of the counter to use to create virtual
      channels. You can specify a list or range of physical
      channels.

    name : str

      The name(s) to assign to the created virtual channel(s). If
      you do not specify a name, NI-DAQmx uses the physical
      channel name as the virtual channel name. If you specify
      your own names for nameToAssignToChannel, you must use the
      names when you refer to these channels in other NI-DAQmx
      functions.

      If you create multiple virtual channels with one call to
      this function, you can specify a list of names separated by
      commas. If you provide fewer names than the number of
      virtual channels you create, NI-DAQmx automatically assigns
      names to the virtual channels.

    source : str

      The terminal to which you connect an external timebase. You
      also can specify a source terminal by using a terminal name.

    idle_state : {'low', 'high'}

      The resting state of the output terminal.

    delay : int

      The number of timebase ticks to wait before generating the
      first pulse.

    low_ticks : int

      The number of timebase ticks that the pulse is low.

    high_ticks : int

      The number of timebase ticks that the pulse is high.

    Returns
    -------

      success_status : bool
    """
    counter = str(counter)
    name = str(name)
    idle_state_map = dict (low=Constants.Val_Low, high=Constants.Val_High)
    idle_state_val = self._get_map_value('idle_state', idle_state_map, idle_state)
    return CALL('CreateCOPulseChanTicks', self, counter, name, source, idle_state_val,
                int32 (delay), int32 (low_ticks), int32 (high_ticks))==0

def create_channel_time(self, counter, name="", units="seconds", idle_state='low',
                        delay = 0, low_time=1, high_time=1):
    """
    Creates channel(s) to generate digital pulses defined by the
    number of timebase ticks that the pulse is at a high state and
    the number of timebase ticks that the pulse is at a low state
    and also adds the channel to the task. The pulses appear on
    the default output terminal of the counter unless you select a
    different output terminal.

    Parameters
    ----------

    counter : str

      The name of the counter to use to create virtual
      channels. You can specify a list or range of physical
      channels.

    name : str

      The name(s) to assign to the created virtual channel(s). If
      you do not specify a name, NI-DAQmx uses the physical
      channel name as the virtual channel name. If you specify
      your own names for nameToAssignToChannel, you must use the
      names when you refer to these channels in other NI-DAQmx
      functions.

      If you create multiple virtual channels with one call to
      this function, you can specify a list of names separated by
      commas. If you provide fewer names than the number of
      virtual channels you create, NI-DAQmx automatically assigns
      names to the virtual channels.

    units : {'seconds'}

      The units in which to specify high and low time.

    idle_state : {'low', 'high'}

      The resting state of the output terminal.

    delay : float

      The amount of time in seconds to wait before generating the
      first pulse.

    low_time : float

      The amount of time the pulse is low, in seconds.

    high_time : float

      The amount of time the pulse is high, in seconds.

    Returns
    -------

      success_status : bool
    """
    counter = str(counter)
    name = str(name)
    units_map = dict (seconds = Constants.Val_Seconds)
    idle_state_map = dict (low=Constants.Val_Low, high=Constants.Val_High)
    units_val = self._get_map_value('units', units_map, units)
    idle_state_val = self._get_map_value('idle_state', idle_state_map, idle_state)
    return CALL('CreateCOPulseChanTime', self, counter, name, units_val, idle_state_val,
                float64 (delay), float64(low_time), float64(high_time))==0



"""
    spyre.Tools.wfm_writer.py
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Tool to write waveform in a .WFM format

    Authors: Alexandre Bourassa, Kevin Miao
    Date: 20/04/2016
"""

import struct
import numpy as _np
from datetime import datetime as _dt
import time as _t
import sys

def array_to_ieee_block(analog, marker1, marker2, prepend_length=True):
    """
        Produces a little-endian 4-byte floating point + 1-byte marker representation of analog
        :param analog: Array of numpy float32
        :param marker1: Array of numpy int8
        :param marker2: Array of numpy int8
        :return: Byte Stream in the WFM format
    """
    num_bytes = 5 * len(analog)
    num_digit = len(str(num_bytes))
    if not marker1.dtype==_np.int8: marker1 = _np.asarray(marker1, dtype=_np.int8)
    if not marker2.dtype==_np.int8: marker2 = _np.asarray(marker2, dtype=_np.int8)
    if not analog.dtype == _np.float32: analog = _np.asarray(analog, dtype=_np.float32)

    points = _np.zeros(len(analog), dtype='<f4, i1')
    points['f1'] = (marker1 + ((marker2)<<1))<<6
    points['f0'] = analog

    #Makes sure that the byteordering is 'little'
    if not sys.byteorder == 'little': points = points.newbyteorder('<')
    bin_all = points.tobytes()

    if prepend_length:  return bytes('#{:d}{:d}'.format(num_digit, num_bytes), encoding='ascii') + bin_all
    else             :  return bin_all

def iee_block_to_array(block):
    """
    Decodes an iee block into three arrays
    :param block: iee formatted block
    :return: analog, marker1, marker2
    """
    block = block.rstrip()

    #Check for a '#'
    if block[0:1] != b'#': raise ValueError("Argument is not a iee formatted block")

    #Check for that there is the correct number of bytes
    num_digit = int(block[1:2])
    num_bytes = int(block[2:2+num_digit])
    block = block[2 + num_digit:]
    if len(block) != num_bytes: raise ValueError("Argument is not a iee formatted block")

    n_points = int(num_bytes/5)
    array = struct.unpack('<'+'fB'*n_points, block)
    analog = _np.array(array[::2])
    marker = _np.array(array[1::2])
    print(marker)
    marker1 = _np.right_shift(_np.bitwise_and(marker, 64), 6)
    marker2 = _np.right_shift(marker, 7)
    return analog, marker1, marker2

class AWG_Record(object):
    def __init__(self, name, data, data_type=None):
        self.name = name
        self.data = data
        if data_type is None:
            if type(data) == str:       self.data_type = 'char'
            elif type(data) == float:   self.data_type = 'double'
            elif type(data) == bytes :  self.data_type = 'bytes'
            else:                       self.data_type = 'short'
        else:
            if not data_type in ['char', 'double', 'long', 'short', 'bytes']: raise Exception("Invalid data type!")
            self.data_type= data_type

    def _get_format_str(self, data, type):
        if type == 'char':      return '%ds'%(len(data)+1)
        elif type == 'double':  return 'd'
        elif type == 'long':    return 'l'
        elif type == 'short':   return 'h'
        elif type == 'bytes':   return '%ds'%(len(data))
        else:   raise Exception("Invalid data type!")

    def _get_length(self, data, type):
        if   type == 'char':   return len(data) + 1
        elif type == 'bytes':  return len(data)
        elif type == 'double': return 8
        elif type == 'long':   return 4
        elif type == 'short':  return 2
        else :raise Exception("Invalid data type!")

    def get_bytes(self):
        fmt = '<ii'+self._get_format_str(self.name, 'char')+self._get_format_str(self.data, self.data_type)
        name_l = self._get_length(self.name, 'char')
        data_l = self._get_length(self.data, self.data_type)
        if self.data_type == 'char':   data = bytes(self.data, encoding='ascii')
        else                       :   data = self.data
        return struct.pack(fmt, name_l, data_l, bytes(self.name, encoding='ascii'), data)


class AWG_File_Writer(object):
    def __init__(self):
        self.records = ([],[],[],[],[],[],[],)
        self.add_record("MAGIC", 5000, 1)
        self.add_record("VERSION", 1, 1)
        self.wfm  = list()
        self.n_seq_lines = 0

    def add_record(self, name, data, group, data_type=None):
        group -= 1
        if not group in range(7): raise Exception("Invalid group!")
        self.records[group].append(AWG_Record(name, data, data_type=data_type))

    def add_waveform(self, name, analog, marker1, marker2):
        if len(self.wfm)>=32000: raise Exception("Maximum 32000 waveform in .AWG file...")
        if len(analog)<250:
            print("WARNING: The AWG will use the software sequencer because this waveform has less than 250 points")
        data = array_to_ieee_block(analog, marker1, marker2, prepend_length=False)
        t = _dt.now()
        tm = [t.year, t.month, t.weekday(), t.day, t.hour, t.minute, t.second, t.microsecond // 1000]
        self.wfm.append(name)
        N = len(self.wfm)
        self.add_record("WAVEFORM_NAME_{}".format(N), name, 5)
        self.add_record("WAVEFORM_TYPE_{}".format(N), 2, 5)
        self.add_record("WAVEFORM_LENGTH_{}".format(N), len(analog), 5, data_type='long')
        self.add_record("WAVEFORM_TIMESTAMP_{}".format(N), struct.pack('<' + 'h' * 8, *tm), 5, data_type='bytes')
        self.add_record("WAVEFORM_DATA_{}".format(N), data, 5)

    def add_sequence_line(self, wfm=("", "", "", ""), use_sub_seq = False, sub_seq_name="",
                          repeat_count=0, wait_for_trigger=False, jump_target=0, goto_target=0):
        if self.n_seq_lines >= 8000: raise Exception("Maximum 8000 lines for main sequence in .AWG file...")
        N = self.n_seq_lines + 1
        if not (len(wfm) == 4): raise Exception("There should be 4 entries in the wfm tuples")
        if not 65536 >= repeat_count >= 0: raise Exception("Maximum of 65536 for repeat_count")
        if not use_sub_seq and wfm[0] == wfm[1] == wfm[2] == wfm[3] == "": raise Exception("At least one channel must have non-empty wfm")
        if use_sub_seq and sub_seq_name=="": raise Exception("sub_seq_name is empty")

        self.add_record('SEQUENCE_WAIT_{}'.format(N), wait_for_trigger, 6)
        self.add_record('SEQUENCE_LOOP_{}'.format(N), repeat_count, 6, data_type='long')
        self.add_record('SEQUENCE_JUMP_{}'.format(N), jump_target, 6)
        self.add_record('SEQUENCE_GOTO_{}'.format(N), goto_target, 6)

        # Add the wfm / subseq

        if use_sub_seq:
            wfm = ("", "", "", "")
        else:
            sub_seq_name = ""

        for i in range(len(wfm)):
            if not use_sub_seq:
                if wfm[i] != "": self.add_record("SEQUENCE_WAVEFORM_NAME_CH_{}_{}".format(i + 1, N), wfm[i], 6)
        self.add_record("SEQUENCE_IS_SUBSEQ_{}".format(N), int(use_sub_seq), 6, data_type='long')
        self.add_record("SEQUENCE_SUBSEQ_NAME_{}".format(N), sub_seq_name, 6)
        self.n_seq_lines += 1

    def add_subseq(self, name):
        ss = Sub_Sequence(name)
        self.records[6].append(ss)
        return ss


    def get_bytes(self):
        ans = list()
        for i in range(len(self.records)):
            group_list = self.records[i]
            if not i == 6:
                ans.extend([entry.get_bytes() for entry in group_list])
            else:
                # Special treatement for subseq group
                subseq_number, cummul_line = 1, 0
                for ss in group_list:
                    if len(ss.lines) != 0:
                        ans += ss.get_bytes(subseq_number,cummul_line)
                        subseq_number += 1
                        cummul_line += len(ss.lines)

        return b''.join(ans)

class Sub_Sequence(object):
    def __init__(self, name):
        self.name = name
        self.lines = list()

    def add_line(self, wfm=("", "", "", ""), repeat_count=1):
        if not 65536 >= repeat_count >= 0: raise Exception("Maximum of 65536 for repeat_count")
        if not (len(wfm) == 4): raise Exception("There should be 4 entries in the wfm tuples")
        if wfm[0] == wfm[1] == wfm[2] == wfm[3] == "": raise Exception("At least one channel must have non-empty wfm")

        self.lines.append([repeat_count, wfm])
        # line = list()
        # n = len(self.lines + 1)
        # line.append(AWG_Record("SUBSEQ_LOOP_{}_{}_{}".format(n,self.o,n), , data_type=data_type))

    def get_bytes(self, subseq_number, cummul_line):
        ans = b''
        u = cummul_line + 1
        o = subseq_number
        t = _dt.now()
        tm = [t.year, t.month, t.weekday(), t.day, t.hour, t.minute, t.second, t.microsecond // 1000]
        rec = [
            AWG_Record("SUBSEQ_NAME_{}".format(o), self.name),
            AWG_Record("SUBSEQ_TIMESTAMP_{}".format(o), struct.pack('<' + 'h' * 8, *tm), data_type='bytes'),
            AWG_Record("SUBSEQ_LENGTH_{}".format(o), len(self.lines), data_type='long')
        ]
        n = 1
        for line in self.lines:
            rec.append(AWG_Record("SUBSEQ_LOOP_{}_{}_{}".format(n,o,u), line[0], data_type='long'))
            wfm = line[1]
            for i in range(len(wfm)):
                    if wfm[i] != "":
                        rec.append(AWG_Record("SUBSEQ_WAVEFORM_NAME_CH_{}_{}_{}_{}".format(i + 1, n, o, u), wfm[i]))
            n += 1
            u += 1
        for entry in rec:
            ans += entry.get_bytes()
        return ans

# -----------------------------------
# DEPRECATED
# -----------------------------------

def create_wfm(analog, marker1, marker2, clock=None):
    """
        Generate the byte stream for a WFM file given 3 arrays (analog, marker1 and marker2)
        :param analog: Array of float
        :param marker1: Array of bool (or 1/0)
        :param marker2: Array of bool (or 1/0)
        :param clock: The clock speed that the waveform should be run at
        :return: Byte Stream in the WFM format
    """
    if not (len(analog) == len(marker1) == len(marker2)):
        raise ValueError('Mismatched analog and marker lengths')
    if max(analog) > 1.0 or min(analog) < -1.0:
        raise ValueError('analog values out of range')

    header = b'MAGIC 1000\r\n'
    trailer = bytes('CLOCK {:1.10E}\r\n'.format(clock), encoding='ascii') if clock is not None else b''
    body =  array_to_iee_block(analog, marker1, marker2)

    return b''.join((header, body, trailer))

class Sequence(object):
    def __init__(self):
        self.seq = []

    def add_line(self, ch1_wfm="", ch2_wfm="", ch3_wfm="", ch4_wfm="", repeat_count=0, wait_for_trigger=False, logic_jump_target=0, finished_goto=0):
        """
        This defines a new sequence line to be added to this SEQ file
        :param ch1_wfm: wfm (or pat) file to be used for CH1 on this line.
        :param ch2_wfm: wfm (or pat) file to be used for CH2 on this line.
        :param ch3_wfm: wfm (or pat) file to be used for CH3 on this line.
        :param ch4_wfm: wfm (or pat) file to be used for CH4 on this line.
        :param repeat_count: Repeat count for the line.  0 is infinity
        :param wait_for_trigger: Specify whether or not to wait for a trigger before running the wfm
        :param logic_jump_target: Line number where to jump upon EVENT IN input or FORCE EVENT triggers.
                                  0 is Off, -1 is next, and -2 is Table-jump
        :param finished_goto: Line to go after current line. 0 is Next. Maximum 8000.
        :return:
        """
        wait_for_trigger = 1 if bool(wait_for_trigger) else 0
        line = '"{}","{}","{}","{}",{},{},{},{},{}\r\n'.format(ch1_wfm, ch2_wfm, ch3_wfm, ch4_wfm,int(repeat_count),
                                                           wait_for_trigger, 0, int(logic_jump_target), finished_goto)
        self.seq.append(line)

    def verify_line(self, line):
        line = line.strip()
        args = line.split(",")
        print(line)
        print(args)
        if len(args) != 9: raise Exception("The number of paramter in the line <{}> is incorrect".format(line))
        if args[0]==args[1]==args[2]==args[3]=="": raise Exception("At least one channel must have non-empty wfm")
        if not 0<=int(args[4])<=65536: raise Exception("Invalid repeat_counts (must be 0 for infinity or [1,65536])")
        if not args[5] in ["0","1"]: raise Exception("wait_for_trigger must be 0 or 1")
        if not args[6] == "0": raise Exception("goto_one is not implemented and therefore must be set to 0")
        if not -2<=int(args[7])<=len(self.seq): raise Exception("Invalid logic_jump_target argument (must be in [-2, N] where N is the number of line in the sequence)")
        if not 0<=int(args[8])<=len(self.seq): raise Exception("Invalid finnished_goto argument (must be in [0, N] where N is the number of line in the sequence)")

    def get_str(self):
        s = "MAGIC 3004A\r\nLINES {}".format(len(self.seq))
        if len(self.seq)>8000: raise Exception("More than 8000 lines may not work...")
        for line in self.seq:
            self.verify_line(line)
            s+= line
        return s

    def get_bytes(self):
        return bytes(self.get_str() ,encoding='ascii')



"""
    lantz.drivers.tektronix.awg5014c
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Drivers for the AWG5014C using RAW SOCKETS

    Authors: Alexandre Bourassa
    Date: 20/04/2016
"""
import numpy as _np

from lantz import Feat, DictFeat, Action
from lantz.feat import MISSING
from lantz.errors import InstrumentError
from lantz.messagebased import MessageBasedDriver

import ftplib as _ftp
import re as _re
import os as _os
import time as _t


from lantz.drivers.tektronix.awg5014c_tools import AWG_File_Writer, create_wfm, iee_block_to_array, array_to_ieee_block, Sequence
import lantz.drivers.tektronix.awg5014c_constants as _cst

class AWG5014C(MessageBasedDriver):
    """E8364B Network Analyzer
    """

    DEFAULTS = {'COMMON': {'write_termination': '\r\n',
                           'read_termination': '\r\n'}}

    def __init__(self, resource_name, *args, **kwargs):
        super(AWG5014C, self).__init__(resource_name, *args, **kwargs)
        self.ip = _re.findall("[0-9]+.[0-9]+.[0-9]+.[0-9]+", resource_name)[0]
        # self.ftp = _ftp.FTP(ip)
        # self.ftp.login()

    def finalize(self):
        pass

    @Feat(read_once=True)
    def idn(self):
        return self.query('*IDN?')

    @Action()
    def get_waveform_data(self, name):
        self.write('WLIS:WAV:DATA? "{}"'.format(name))
        data = self.resource.read_raw()
        return iee_block_to_array(data)

    @Action()
    def set_waveform_data(self, name, analog, marker1, marker2, start_index=None, size=None):
        """ Sets the data for waveform <name>.
            analog should be an array of float and marker 1 and 2 an array of bool or 0/1
            analog, marker1 and marker2 should have the same dimensions
        """
        data = array_to_ieee_block(analog, marker1, marker2)
        cmd = bytes('WLIS:WAV:DATA "{}",'.format(name), encoding='ascii')
        if not start_index is None:
            cmd += bytes('{},'.format(start_index), encoding='ascii')
            if not size is None:
                cmd += bytes('{},'.format(size), encoding='ascii')
        term = bytes(self.resource.write_termination, encoding='ascii')

        cmd += data + term
        self.log_debug('Writing {!r}', cmd)
        self.resource.write_raw(cmd+data+term)

    @Action()
    def create_new_waveform(self, name, size, type='REAL'):
        """ Create a new waveform with a given name and size (in number of points).
            type can be either REAL or INT, but only REAL is supported for now
        """
        self.write('WLIS:WAV:NEW "{}" {}'.format(name, size, type))

    @Action()
    def delete_waveform(self, name):
        """Delete wfm <name>.  If <name>=='ALL', deletes all user defined waveform"""
        if name == 'ALL': self.write('WLIS:WAV:DEL ALL')
        else: self.write('WLIS:WAV:DEL "{}"'.format(name))



    # ----------------------------------------------------
    # Hard Drive navigation method
    # ----------------------------------------------------
    VALID_DRIVE = {"MAIN", "FLOP", "NET1", "NET2", "NET3"}

    @Action()
    def cd(self, dir):
        self.write('MMEM:CDIR "{}"'.format(dir))

    @Action()
    def ls(self, verbose=True):
        #Query the current directory and the content
        dir = self.query("MMEM:CDIR?").strip('"')
        content = self.query("MMEM:CAT?").split(',"')
        used, avail = map(int, content.pop(0).split(","))

        dirs = list()
        files = list()
        for item in content:
            item = item.strip('"')
            name, isDir, size = item.split(',')
            if isDir=='DIR': dirs.append(name)
            else           : files.append((name, size))

        # Build and print an answer (or return it
        if verbose:
            spaces_size = 30
            ans =  "{} ({:.2f}% full):\r\n".format(dir, (used / (used + avail)))
            for d in dirs: ans += '\t|_ '+d+(' '*(spaces_size-len(d)))+'DIR\n'
            for f in files: ans += '\t|_ '+f[0]+(' '*(spaces_size-len(f[0])))+f[1]+'\n'
            print(ans)
        else:
            return (dir, used, avail), dirs, files

    @Action()
    def mkdir(self, dir_name, drive="MAIN"):
        if not drive in self.VALID_DRIVE: raise Exception("Invalid drive!")
        self.write(':MMEM:MDIR "{}", "{}"'.format(dir_name, drive))

    @Action()
    def select_drive(self, drive):
        if not drive in self.VALID_DRIVE: raise Exception("Invalid drive!")
        self.write(':MMEM:MSIS "{}"'.format(drive))

    @Action()
    def get_current_drive(self):
        print(self.query(':MMEM:MSIS?'))

    #This might not work
    @Action()
    def mv(self, source_file_path, dest_file_path, source_drive="MAIN", dest_drive="MAIN"):
        if not source_drive in self.VALID_DRIVE: raise Exception("Invalid source drive!")
        if not dest_drive in self.VALID_DRIVE: raise Exception("Invalid destination drive!")
        self.write(':MMEM:MOVE "{}","{}","{}","{}"'.format(source_file_path, source_drive, dest_file_path, dest_drive))

    # This might not work
    @Action()
    def cp(self, source_file_path, dest_file_path, source_drive="MAIN", dest_drive="MAIN"):
        if not source_drive in self.VALID_DRIVE: raise Exception("Invalid source drive!")
        if not dest_drive in self.VALID_DRIVE: raise Exception("Invalid destination drive!")
        self.write(':MMEM:COPY "{}","{}","{}","{}"'.format(source_file_path, source_drive, dest_file_path, dest_drive))

    def upload_file(self, local_filename, remote_filename, print_progress=True):
        self.ftp = _ftp.FTP(self.ip)
        self.ftp.login()
        if print_progress:
            FTP_Upload(self.ftp,local_filename, remote_filename)
        else:
            self.ftp.storbinary('STOR ' + remote_filename, open(local_filename, 'rb'), blocksize=1024)
        self.ftp.quit()

    @Action()
    def force_jump(self, line_number):
        self.write("SEQUENCE:JUMP:IMMEDIATE {}".format(line_number))

    @Action()
    def trigger(self):
        self.write("*TRG")

    # ----------------------------------------------------
    # Source
    # ----------------------------------------------------
    @Action()
    def load_waveform(self, filename, drive="MAIN"):
        """This clear the user defined wlist and loads either a WFM, SEQ or PAT to it"""
        if not drive in self.VALID_DRIVE: raise Exception("Invalid drive!")
        self.write('SOURCE1:FUNCTION:USER "{}","{}"'.format(filename, drive))

    @Action()
    def load_awg_file(self, filename):
        self.write('AWGCONTROL:SRESTORE "{}"'.format(filename))



class FTP_Upload():
    def __init__(self, ftp, local_filename, remote_filename):

        block_size = 1024
        total_size = _os.path.getsize(local_filename)
        self.written_size = 0
        self.last_time = _t.time()
        self.last_percent = 0

        def callback(block):
            self.written_size += block_size
            time = _t.time()
            percent = (self.written_size / total_size)*100
            #print(time - self.last_time > 1,percent - self.last_percent  > 0.1)
            if time - self.last_time > 1 and percent - self.last_percent  > 0.1:
                self.last_time = time
                self.last_percent = percent
                print('{:.2f}%'.format(percent))
        print('Uploading "{}" to remote destination "{}"'.format(local_filename, remote_filename))
        ftp.storbinary('STOR ' + remote_filename, open(local_filename, 'rb'), blocksize=block_size, callback=callback)


# -----------------------------------
# DEBUGING AND TESTING
# -----------------------------------

def test_awg_file(awg):
    a = AWG_File_Writer()

    # Add waveform
    analog, m1, m2 = generate_test_sequence1()
    a.add_waveform('test', analog, m1, m2)
    analog, m1, m2 = generate_test_sequence2()
    a.add_waveform('hey', analog, m1, m2)

    # Add sub-sequence
    s = a.add_subseq("pySeq")
    s.add_line(wfm=("hey","hey","hey",""), repeat_count= 2)
    s.add_line(wfm=("test", "test", "test", ""), repeat_count=3)
    s.add_line(wfm=("hey", "hey", "hey", ""), repeat_count=4)

    # Add main sequence lines
    a.add_sequence_line(wfm=("test", "hey", "test", ""), use_sub_seq=False, sub_seq_name="", repeat_count=100,
                        wait_for_trigger=False, jump_target=0, goto_target=0)
    a.add_squence_line(use_sub_seq=True, sub_seq_name="pySeq", repeat_count=1)

    # Add some more records
    a.add_record("RUN_MODE", _cst.ERunMode.RunMode_SEQUENCE, 2)
    a.add_record(_cst.Commands.CS_SAMPLING_RATE, 500e6, 2)

    with open("test.awg", "wb") as f:
        f.write(a.get_bytes())
    awg.upload_file('test.awg', 'test.awg')
    awg.load_awg_file('test.awg')


def generate_test_sequence1():
    """Generate a test sequence with two pulse
     one in between sequence and one within
    """
    #Create the wfm
    m1 = _np.zeros(1024, dtype=_np.int32)
    m2 = _np.ones(1024, dtype=_np.int32)
    m2[0::2] = _np.zeros(512)
    analog = _np.zeros(1024)
    m1[-1] = 1
    m1[1] = 1
    return analog, m1, m2

def generate_test_sequence2():
    """Generate a test sequence with two pulse
     one in between sequence and one within
    """
    #Create the wfm
    m1 = _np.ones(1024, dtype=_np.int32)
    m2 = _np.zeros(1024, dtype=_np.int32)
    analog = _np.random.rand(1024)
    return analog, m1, m2




if __name__=='__main__':
    awg = AWG5014C('TCPIP0::192.168.1.104::4444::SOCKET')
    awg.initialize()


# -*- coding: utf-8 -*-
"""
    lantz.drivers.stanford.dg645
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    Implementation of DG645 digital delay generator
    Author: Kevin Miao & Berk Diler
    Date: 1/14/2016 & 11/08/2017
"""

from collections import OrderedDict
from lantz import Action, Feat, DictFeat, Q_
from lantz.messagebased import MessageBasedDriver

class DGException(Exception):
    pass

class DG645(MessageBasedDriver):

    DEFAULTS = {
        'COMMON': {
            'write_termination': '\r\n',
            'read_termination': '\r\n',
        }
    }

    ERRORS = OrderedDict([
                   ('NO ERROR', 0),
                   ('EXECUTION ERROR: Illegal Value. \n'
                    'Parameter was out of range.', 10),
                   ('EXECUTION ERROR: Illegal Mode. \n'
                    'The action is illegal in the current mode. This might happen, for instance, if a single shot '
                    'is requested when the trigger source is not single shot.', 11),
                   ('EXECUTION ERROR: Illegal Delay. \n'
                    'The requested delay is out of range.', 12),
                   ('EXECUTION ERROR: Illegal Link. \n'
                    'The requested delay linkage is illegal.', 13),
                   ('EXECUTION ERROR: Recall Failed. \n'
                    'The recall of instrument settings from nonvolatile storage failed. '
                    'The instrument settings were invalid. ', 14),
                   ('EXECUTION ERROR: Not Allowed \n'
                    'The requested action is not allowed because the instrument is locked by another interface.', 15),
                   ('EXECUTION ERROR: Failed Self Test. \n'
                    'The DG645 self test failed.', 16),
                   ('EXECUTION ERROR: Failed Auto Calibration \n'
                    'The DG645 auto calibration failed.', 17),
                   ('QUERY ERROR: Lost Data \n'
                    'Data in the output buffer was lost. This occurs if the output buffer overflows, or if a'
                    'communications error occurs and data in output buffer is discarded. ', 30),
                   ('QUERY ERROR: No Listener \n'
                    'This is a communications error that occurs if the DG645 is addressed to talk on the GPIB '
                    'bus, but there are no listeners. The DG645 discards any pending output. ', 32),
                   ('PARSING ERROR: Illegal Command. \n'
                    'The command syntax used was illegal. A command is normally a sequence of four letters, '
                    'or a ‘*’ followed by three letters.', 110),
                   ('PARSING ERROR: Undefined Command. \n'
                    'The specified command does not exist.', 111),
                   ('PARSING ERROR: Illegal Query. \n'
                    'The specified command does not permit queries.', 112),
                   ('PARSING ERROR: Illegal Set. \n'
                    'The specified command can only be queried.', 113),
                   ('PARSING ERROR: Null Parameter. \n'
                    'The parser detected an empty parameter.', 114),
                   ('PARSING ERROR: Extra Parameters \n'
                    'The parser detected more parameters than allowed by the command.', 115),
                   ('PARSING ERROR: Missing Parameters. \n'
                    'The parser detected missing parameters required by the command.', 116),
                   ('PARSING ERROR: Parameter Overflow \n'
                    'The buffer for storing parameter values overflowed. This probably indicates a syntax error.', 117),
                   ('PARSING ERROR: Invalid Floating Point Number \n'
                    'The parser expected a floating point number, but was unable to parse it.', 118),
                   ('PARSING ERROR: Invalid Integer  \n'
                    'The parser expected an integer, but was unable to parse it.', 120),
                   ('PARSING ERROR: Integer Overflow \n'
                    'A parsed integer was too large to store correctly.', 121),
                   ('PARSING ERROR: Invalid Hexadecimal \n'
                    'The parser expected hexadecimal characters but was unable to parse them.', 122),
                   ('PARSING ERROR: Syntax Error \n'
                    'The parser detected a syntax error in the command.', 126),
                   ('COMMUNICATION ERROR: Communication Error \n'
                    'A communication error was detected. This is reported if the hardware detects a framing, '
                    'or parity error in the data stream. ', 170),
                   ('COMMUNICATION ERROR: Over run \n '
                    'The input buffer of the remote interface overflowed. All data in both the input and output '
                    'buffers will be flushed. ', 171),
                   ('VERY SERIOUS ERROR: Too Many Errors  \n'
                    'You destroyed the instrument, throw it in the garbage. \n'
                    'Just kidding. Check your errors though.\n'
                    'Probably the error buffer is full. Subsequent errors have been dropped. ', 254),
    ])

    CHANNELS = OrderedDict([
                   ('T0', 0),
                   ('T1', 1),
                   ('A', 2),
                   ('B', 3),
                   ('C', 4),
                   ('D', 5),
                   ('E', 6),
                   ('F', 7),
                   ('G', 8),
                   ('H', 9)
                   ])

    OUTPUT_CHANNELS = OrderedDict([
                   ('T0', 0),
                   ('AB', 1),
                   ('CD', 2),
                   ('EF', 3),
                   ('GH', 4)
                   ])

    TRIGGER_CHANNELS = OrderedDict([
                   ('Internal', 0),
                   ('External Rising Edge', 1),
                   ('External Falling Edge', 2),
                   ('Single Shot External Rising Edge', 3),
                   ('Single Shot External Falling Edge', 4),
                   ('Single Shot', 5),
                   ('Line', 6)
                   ])

    @Feat(read_once=True)
    def idn(self):
        self.clear_errors()
        return self.query('*IDN?')

    @Action()
    def reset(self):
        self.clear_errors()
        self.write('*RST')

    # Error Commands
    @Feat(values=ERRORS)
    def error(self):
        return int(self.query("LERR?"))

    @Action()
    def clear_errors(self):
        err = self.error
        while err != "NO ERROR":
            raise DGException(err)
            err = self.error


    # Delay Commands

    @DictFeat(keys = CHANNELS, units="s")
    def delay(self, channel):
        retval = self.query('DLAY? {}'.format(channel))
        linked, offset = retval.split(',')
        self.clear_errors()
        return float(offset)

    @delay.setter
    def delay(self, channel, value):
        retval = self.query('DLAY? {}'.format(channel))
        linked, offset = retval.split(',')
        new_str = 'DLAY {},'+str(linked)+',{:1.12e}'
        self.write(new_str.format(channel,value))
        self.clear_errors()

    @DictFeat(keys = CHANNELS)
    def reference(self, channel):
        retval = self.query('DLAY? {}'.format(channel))
        linked, offset = retval.split(',')
        self.clear_errors()
        return list(self.CHANNELS.keys())[int(linked)]

    @reference.setter
    def reference(self, channel, reference):
        new_ref = self.CHANNELS[reference]
        retval = self.query('DLAY? {}'.format(channel))
        linked, offset = retval.split(',')
        new_str = 'DLAY {},{:d},' + offset
        args = channel, new_ref
        self.write(new_str.format(*args))
        self.clear_errors()

    @DictFeat(keys = OUTPUT_CHANNELS, units="V", limits=(0.5,5.0))
    def amplitude(self, channel):
        retval = self.query('LAMP?{}'.format(channel))
        self.clear_errors()
        return float(retval)

    @amplitude.setter
    def amplitude(self, channel, value):
        self.write("LAMP {},{:1.3e}".format(channel,value))
        self.clear_errors()

    @DictFeat(keys = OUTPUT_CHANNELS, units="V", limits=(-2.,2.))
    def offset(self, channel):
        retval = self.query('LOFF?{}'.format(channel))
        self.clear_errors()
        return float(retval)

    @offset.setter
    def offset(self, channel, value):
        self.write("LOFF {},{:1.2e}".format(channel,value))
        self.clear_errors()

    @DictFeat(keys = OUTPUT_CHANNELS, values={"POS" : 1, "NEG" : 0})
    def polarity(self,channel):
        answ = float(self.query("LPOL? {}".format(channel)))
        self.clear_errors()
        return answ

    @polarity.setter
    def polarity(self,channel,pol):
        self.write("LPOL {},{}".format(channel,pol))
        self.clear_errors()

    # Trigger Commands

    @Feat(values = TRIGGER_CHANNELS)
    def trigger_source(self):
        answ = float(self.query('TSRC?'))
        self.clear_errors()
        return answ

    @trigger_source.setter
    def trigger_source(self,src):
        self.write('TSRC {:d}'.format(src))
        self.clear_errors()

    @Feat(units = "Hz")
    def trigger_rate(self):
        answ = float(self.query("TRAT?"))
        self.clear_errors()
        return answ

    @trigger_rate.setter
    def trigger_rate(self,rate):
        self.write("TRAT {:d}".format(rate))
        self.clear_errors()

    @Feat(units="V", limits=(0,3.5))
    def trigger_level(self):
        answ = float(self.query("TLVL?"))
        self.clear_errors()
        return answ

    @trigger_level.setter
    def trigger_level(self, lvl):
        self.write("TLVL {:1.2e}".format(lvl))
        self.clear_errors()

    # BURST Commands

    @Feat(limits=(1, 2 ** 32 - 1))
    def burst_count(self):
        answ = self.query('BURC?')
        self.clear_errors()
        return answ

    @burst_count.setter
    def burst_count(self, value):
        self.write('BURC {:d}'.format(value))
        self.clear_errors()

    @Feat(units='s')
    def burst_delay(self):
        answ =self.query('BURD?')
        self.clear_errors()
        return answ

    @burst_delay.setter
    def burst_delay(self, value):
        self.write('BURD {:1.12e}'.format(value))
        self.clear_errors()

    @Feat(values={True: '1', False: '0'})
    def burst_mode(self):
        answ = self.query('BURM?')
        self.clear_errors()
        return answ

    @burst_mode.setter
    def burst_mode(self, value):
        self.write('BURM {:s}'.format(value))
        self.clear_errors()

    @Feat(units='s', limits=(100e-9, (2 ** 32 - 1) * 1e-8, 10e-9))
    def burst_period(self):
        answ = self.query('BURP?')
        self.clear_errors()
        return answ

    @burst_period.setter
    def burst_period(self, value):
        self.write('BURP {:1.12e}')
        self.clear_errors()

    @Feat(values={True: '1', False: '0'})
    def burst_config(self):
        answ = self.query('BURT?')
        self.clear_errors()
        return answ

    @burst_config.setter
    def burst_config(self, value):
        self.write('BURT {:s}'.format(value))
        self.clear_errors()


"""
This constant file was automatically generated by a quick script I wrote for the enum part and h2py for the constant part.  It aims
to help out remembering the enum values and constants, but probably as bugs... please refer to the actual documentation for the correct values (if something is not working)
and feel free to correct the values and submit a pull request.
"""


class EWfmItem:
	WfmItem_MARKER1 = 1
	WfmItem_MARKER2 = 2
	WfmItem_DIGITAL0 = 3
	WfmItem_DIGITAL1 = 4
	WfmItem_DIGITAL2 = 5
	WfmItem_DIGITAL3 = 6
	WfmItem_DIGITAL4 = 7
	WfmItem_DIGITAL5 = 8
	WfmItem_DIGITAL6 = 9
	WfmItem_DIGITAL7 = 10
	WfmItem_DIGITAL8 = 11
	WfmItem_DIGITAL9 = 12
	WfmItem_DIGITAL10 = 13
	WfmItem_DIGITAL11 = 14
	WfmItem_DIGITAL12 = 15
	WfmItem_DIGITAL13 = 16
class ELogicSignature:
	LogicSignature_0 = 1
	LogicSignature_1 = 2
	LogicSignature_MIX = 3
class EStdWfmType:
	StdWfmType_SQUARE = 1
	StdWfmType_TRIANGLE = 2
	StdWfmType_RAMP = 3
	StdWfmType_GAUSSIAN_NOISE = 4
class EPointDrawInterpolationType:
	PointDrawInterpolationType_SPLINE = 1
class EDualWfmMathType:
	MATH_SUB = 1
class EMarkDensity:
	MARK_DENSITY_1_4 = 1
	MARK_DENSITY_1_2 = 2
	MARK_DENSITY_3_4 = 3
class ESeqItem:
	ITEM_WFM_CH2 = 1
	ITEM_WFM_CH3 = 2
	ITEM_WFM_CH4 = 3
	ITEM_WAIT = 4
	ITEM_LOOP = 5
	ITEM_JUMP_TO = 6
	ITEM_GO_TO = 7
class ECalDiagType:
	CAL_DIAG_TYPE_YIG = 1
	CAL_DIAG_TYPE_CLOCKPOWER = 2
	CAL_DIAG_TYPE_IPC = 3
	CAL_DIAG_TYPE_DTIMING = 4
	CAL_DIAG_TYPE_ANALOG = 5
	CAL_DIAG_TYPE_MARKER = 6
	CAL_DIAG_TYPE_MDELAY = 7
	CAL_DIAG_TYPE_MCPOINT = 8
	CAL_DIAG_TYPE_SKEW = 9
class EUserCalDiagType:
	USER_CALDIAG_TYPE_D_AWG_REGISTER = 1
	USER_CALDIAG_TYPE_D_AWG_WFM_MEM_CH1 = 2
	USER_CALDIAG_TYPE_D_AWG_WFM_MEM_CH2 = 3
	USER_CALDIAG_TYPE_D_AWG_WFM_MEM_CH3 = 4
	USER_CALDIAG_TYPE_D_AWG_WFM_MEM_CH4 = 5
	USER_CALDIAG_TYPE_D_CLOCK_REGISTER = 6
	USER_CALDIAG_TYPE_D_CLOCK_PLL = 7
	USER_CALDIAG_TYPE_D_OUTPUT1_REGISTER = 8
	USER_CALDIAG_TYPE_D_OUTPUT1_ANALOG = 9
	USER_CALDIAG_TYPE_D_OUTPUT1_MARKER = 10
	USER_CALDIAG_TYPE_D_OUTPUT2_REGISTER = 11
	USER_CALDIAG_TYPE_D_OUTPUT2_ANALOG = 12
	USER_CALDIAG_TYPE_D_OUTPUT2_MARKER = 13
	USER_CALDIAG_TYPE_C_LEVEL_ANALOG_CH1 = 14
	USER_CALDIAG_TYPE_C_LEVEL_MARKER_CH1 = 15
	USER_CALDIAG_TYPE_C_LEVEL_ANALOG_CH2 = 16
	USER_CALDIAG_TYPE_C_LEVEL_MARKER_CH2 = 17
	USER_CALDIAG_TYPE_C_LEVEL_ANALOG_CH3 = 18
	USER_CALDIAG_TYPE_C_LEVEL_MARKER_CH3 = 19
	USER_CALDIAG_TYPE_C_LEVEL_ANALOG_CH4 = 20
	USER_CALDIAG_TYPE_C_LEVEL_MARKER_CH4 = 21
	USER_CALDIAG_TYPE_D_AWG_DTIMING = 22
	USER_CALDIAG_TYPE_D_OUTPUT1_ANALOG_2 = 23
class EClockSource:
	ClockSource_INTERNAL = 1
	ClockSource_EXTERNAL = 2
class EReferenceSource:
	ReferenceSource_INTERNAL = 1
	ReferenceSource_EXTERNAL = 2
class EReferenceType:
	ReferenceType_Fixed = 1
	ReferenceType_Variable = 2
class EReferenceClockFrequency:
	ReferenceClockFrequency_10MHZ = 1
	ReferenceClockFrequency_20MHZ = 2
	ReferenceClockFrequency_100MHZ = 3
class EDividerRate:
	DividerRate_DIV_1 = 1
	DividerRate_DIV_2 = 2
	DividerRate_DIV_4 = 3
	DividerRate_DIV_8 = 4
	DividerRate_DIV_16 = 5
	DividerRate_DIV_32 = 6
	DividerRate_DIV_64 = 7
	DividerRate_DIV_128 = 8
	DividerRate_DIV_256 = 9
class EExternalReferenceStatus:
	ExternalReferenceStatus_INPUT_OK = 1
	ExternalReferenceStatus_INPUT_MISSING = 2
	ExternalReferenceStatus_INPUT_OUT_OF_RANGE = 3
	ExternalReferenceStatus_INPUT_UNSTABLE = 4
class EExternalOscillatorStatus:
	ExternalOscillatorStatus_INPUT_OK = 1
	ExternalOscillatorStatus_INPUT_MISSING = 2
	ExternalOscillatorStatus_INPUT_OUT_OF_RANGE = 3
	ExternalOscillatorStatus_INPUT_UNSTABLE = 4
class ETriggerSource:
	TriggerSource_EXTERNAL = 1
	TriggerSource_INTERNAL = 2
class ETriggerInputImpedance:
	TriggerInputImpedance_50OHM = 1
	TriggerInputImpedance_1KOHM = 2
class ETriggerInputSlope:
	TriggerInputSlope_POL_POSITIVE = 1
	TriggerInputSlope_POL_NEGATIVE = 2
class ETriggerInputPolarity:
	TriggerInputPolarity_POSITIVE = 1
	TriggerInputPolarity_NEGATIVE = 2
class EEventInputImpedance:
	EventInputImpedance_50OHM = 1
	EventInputImpedance_1KOHM = 2
class EEventInputPolarity:
	EventInputPolarity_POSITIVE = 1
	EventInputPolarity_NEGATIVE = 2
class EJumpTiming:
	JumpTiming_SYNC = 1
	JumpTiming_ASYNC = 2
class EAnalogInputMethod:
	AnalogInputMethod_IMAmplitudeOffset = 1
	AnalogInputMethod_IMHighLow = 2
class EDacResolution:
	DacResolution_8 = 1
	DacResolution_10 = 2
	DacResolution_14 = 3
class EFilter:
	Filter_20MHZ = 1
	Filter_50MHZ = 2
	Filter_100MHZ = 3
	Filter_200MHZ = 4
	Filter_THROUGH = 5
class EMarkerInputMethod:
	MarkerInputMethod_IMAmplitudeOffset = 1
	MarkerInputMethod_IMHighLow = 2
class EChannelCoupling:
	ChannelCoupling_Off = 1
	ChannelCoupling_Pair = 2
	ChannelCoupling_All = 3
class ERunMode:
	RunMode_CONTINUOUS = 1
	RunMode_TRIGGERED = 2
	RunMode_GATED = 3
	RunMode_SEQUENCE = 4
class ESequencerState:
	SequencerState_STOPPED = 1
	SequencerState_RUNNING = 2
	SequencerState_WAIT_FOR_TRIGGER = 3
class ESequencerType:
	SequencerType_HW = 1
	SequencerType_SW = 2
class EWaitValue:
	WaitValue_FIRST = 1
	WaitValue_LAST = 2
class EJumpTargetType:
	JumpTargetType_OFF = 1
	JumpTargetType_INDEX = 2
	JumpTargetType_NEXT = 3
class EWaveformType:
	WaveformType_INTEGER = 1
	WaveformType_REAL = 2
class EWfmDacResolution:
	WfmDacResolution_8 = 1
	WfmDacResolution_10 = 2
	WfmDacResolution_14 = 3
class EOverHeat:
	OverHeat_None = 1
	OverHeat_Message = 2
	OverHeat_Shutdown = 3
class EDigitalInputMethod:
	DigitalInputMethod_IMAmplitudeOffset = 1
	DigitalInputMethod_IMHighLow = 2
class ENormalizationType:
	NormalizationType_NONE = 1
	NormalizationType_FULL_SCALE = 2
	NormalizationType_ZERO_REFERENCE = 3
class EPhaseDelayInputMethod:
	PhaseDelayInputMethod_PHASE = 1
	PhaseDelayInputMethod_DELAY_IN_TIME = 2
class ETeklinkConnectionType:
	TeklinkConnectionType_NONE = 1
	TeklinkConnectionType_HUB = 2
	TeklinkConnectionType_INSTRUMENT = 3
class ETeklinkCpState:
	TeklinkCpState_NOTRUNNING = 1
	TeklinkCpState_RUNNING = 2
	TeklinkCpState_PENDING = 3
class EJumpMode:
	JumpMode_EVENT = 1
	JumpMode_TABLE = 2

class Commands:
	CS_MAGIC = "MAGIC"
	CS_VERSION = "VERSION"
	CS_SAMPLING_RATE = "SAMPLING_RATE"
	CS_REPETITION_RATE = "REPETITION_RATE"
	CS_HOLD_REPETITION_RATE = "HOLD_REPETITION_RATE"
	CS_CLOCK_SOURCE = "CLOCK_SOURCE"
	CS_REFERENCE_SOURCE = "REFERENCE_SOURCE"
	CS_EXTERNAL_REFERENCE_TYPE = "EXTERNAL_REFERENCE_TYPE"
	CS_REFERENCE_CLOCK_FREQUENCY_SELECTION = "REFERENCE_CLOCK_FREQUENCY_SELECTION"
	CS_REFERENCE_MULTIPLIER_RATE = "REFERENCE_MULTIPLIER_RATE"
	CS_DIVIDER_RATE = "DIVIDER_RATE"
	CS_TRIGGER_SOURCE = "TRIGGER_SOURCE"
	CS_INTERNAL_TRIGGER_RATE = "INTERNAL_TRIGGER_RATE"
	CS_TRIGGER_INPUT_IMPEDANCE = "TRIGGER_INPUT_IMPEDANCE"
	CS_TRIGGER_INPUT_SLOPE = "TRIGGER_INPUT_SLOPE"
	CS_TRIGGER_INPUT_POLARITY = "TRIGGER_INPUT_POLARITY"
	CS_TRIGGER_INPUT_THRESHOLD = "TRIGGER_INPUT_THRESHOLD"
	CS_EVENT_INPUT_IMPEDANCE = "EVENT_INPUT_IMPEDANCE"
	CS_EVENT_INPUT_POLARITY = "EVENT_INPUT_POLARITY"
	CS_EVENT_INPUT_THRESHOLD = "EVENT_INPUT_THRESHOLD"
	CS_JUMP_TIMING = "JUMP_TIMING"
	CS_INTERLEAVE = "INTERLEAVE"
	CS_ZEROING = "ZEROING"
	CS_INTERLEAVE_ADJ_PHASE = "INTERLEAVE_ADJ_PHASE"
	CS_INTERLEAVE_ADJ_AMPLITUDE = "INTERLEAVE_ADJ_AMPLITUDE"
	CS_COUPLING = "COUPLING"
	CS_RUN_MODE = "RUN_MODE"
	CS_WAIT_VALUE = "WAIT_VALUE"
	CS_RUN_STATE = "RUN_STATE"
	CS_EVENT_JUMP_MODE = "EVENT_JUMP_MODE"
	CS_TABLE_JUMP_STROBE = "TABLE_JUMP_STROBE"
	CS_TABLE_JUMP_DEFINITION = "TABLE_JUMP_DEFINITION"
	CS_DAC_RESOLUTION_1 = "DAC_RESOLUTION_1"
	CS_OUTPUT_WAVEFORM_NAME_1 = "OUTPUT_WAVEFORM_NAME_1"
	CS_CHANNEL_STATE_1 = "CHANNEL_STATE_1"
	CS_ANALOG_DIRECT_OUTPUT_1 = "ANALOG_DIRECT_OUTPUT_1"
	CS_ANALOG_FILTER_1 = "ANALOG_FILTER_1"
	CS_ANALOG_METHOD_1 = "ANALOG_METHOD_1"
	CS_ANALOG_AMPLITUDE_1 = "ANALOG_AMPLITUDE_1"
	CS_ANALOG_OFFSET_1 = "ANALOG_OFFSET_1"
	CS_ANALOG_HIGH_1 = "ANALOG_HIGH_1"
	CS_ANALOG_LOW_1 = "ANALOG_LOW_1"
	CS_MARKER1_SKEW_1 = "MARKER1_SKEW_1"
	CS_MARKER1_METHOD_1 = "MARKER1_METHOD_1"
	CS_MARKER1_AMPLITUDE_1 = "MARKER1_AMPLITUDE_1"
	CS_MARKER1_OFFSET_1 = "MARKER1_OFFSET_1"
	CS_MARKER1_HIGH_1 = "MARKER1_HIGH_1"
	CS_MARKER1_LOW_1 = "MARKER1_LOW_1"
	CS_MARKER2_SKEW_1 = "MARKER2_SKEW_1"
	CS_MARKER2_METHOD_1 = "MARKER2_METHOD_1"
	CS_MARKER2_AMPLITUDE_1 = "MARKER2_AMPLITUDE_1"
	CS_MARKER2_OFFSET_1 = "MARKER2_OFFSET_1"
	CS_MARKER2_HIGH_1 = "MARKER2_HIGH_1"
	CS_MARKER2_LOW_1 = "MARKER2_LOW_1"
	CS_DIGITAL_METHOD_1 = "DIGITAL_METHOD_1"
	CS_DIGITAL_AMPLITUDE_1 = "DIGITAL_AMPLITUDE_1"
	CS_DIGITAL_OFFSET_1 = "DIGITAL_OFFSET_1"
	CS_DIGITAL_HIGH_1 = "DIGITAL_HIGH_1"
	CS_DIGITAL_LOW_1 = "DIGITAL_LOW_1"
	CS_EXTERNAL_ADD_1 = "EXTERNAL_ADD_1"
	CS_PHASE_DELAY_INPUT_METHOD_1 = "PHASE_DELAY_INPUT_METHOD_1"
	CS_PHASE_1 = "PHASE_1"
	CS_DELAY_IN_TIME_1 = "DELAY_IN_TIME_1"
	CS_DELAY_IN_POINTS_1 = "DELAY_IN_POINTS_1"
	CS_CHANNEL_SKEW_1 = "CHANNEL_SKEW_1"
	CS_DAC_RESOLUTION_2 = "DAC_RESOLUTION_2"
	CS_OUTPUT_WAVEFORM_NAME_2 = "OUTPUT_WAVEFORM_NAME_2"
	CS_CHANNEL_STATE_2 = "CHANNEL_STATE_2"
	CS_ANALOG_DIRECT_OUTPUT_2 = "ANALOG_DIRECT_OUTPUT_2"
	CS_ANALOG_FILTER_2 = "ANALOG_FILTER_2"
	CS_ANALOG_METHOD_2 = "ANALOG_METHOD_2"
	CS_ANALOG_AMPLITUDE_2 = "ANALOG_AMPLITUDE_2"
	CS_ANALOG_OFFSET_2 = "ANALOG_OFFSET_2"
	CS_ANALOG_HIGH_2 = "ANALOG_HIGH_2"
	CS_ANALOG_LOW_2 = "ANALOG_LOW_2"
	CS_MARKER1_SKEW_2 = "MARKER1_SKEW_2"
	CS_MARKER1_METHOD_2 = "MARKER1_METHOD_2"
	CS_MARKER1_AMPLITUDE_2 = "MARKER1_AMPLITUDE_2"
	CS_MARKER1_OFFSET_2 = "MARKER1_OFFSET_2"
	CS_MARKER1_HIGH_2 = "MARKER1_HIGH_2"
	CS_MARKER1_LOW_2 = "MARKER1_LOW_2"
	CS_MARKER2_SKEW_2 = "MARKER2_SKEW_2"
	CS_MARKER2_METHOD_2 = "MARKER2_METHOD_2"
	CS_MARKER2_AMPLITUDE_2 = "MARKER2_AMPLITUDE_2"
	CS_MARKER2_OFFSET_2 = "MARKER2_OFFSET_2"
	CS_MARKER2_HIGH_2 = "MARKER2_HIGH_2"
	CS_MARKER2_LOW_2 = "MARKER2_LOW_2"
	CS_DIGITAL_METHOD_2 = "DIGITAL_METHOD_2"
	CS_DIGITAL_AMPLITUDE_2 = "DIGITAL_AMPLITUDE_2"
	CS_DIGITAL_OFFSET_2 = "DIGITAL_OFFSET_2"
	CS_DIGITAL_HIGH_2 = "DIGITAL_HIGH_2"
	CS_DIGITAL_LOW_2 = "DIGITAL_LOW_2"
	CS_EXTERNAL_ADD_2 = "EXTERNAL_ADD_2"
	CS_PHASE_DELAY_INPUT_METHOD_2 = "PHASE_DELAY_INPUT_METHOD_2"
	CS_PHASE_2 = "PHASE_2"
	CS_DELAY_IN_TIME_2 = "DELAY_IN_TIME_2"
	CS_DELAY_IN_POINTS_2 = "DELAY_IN_POINTS_2"
	CS_CHANNEL_SKEW_2 = "CHANNEL_SKEW_2"
	CS_DAC_RESOLUTION_3 = "DAC_RESOLUTION_3"
	CS_OUTPUT_WAVEFORM_NAME_3 = "OUTPUT_WAVEFORM_NAME_3"
	CS_CHANNEL_STATE_3 = "CHANNEL_STATE_3"
	CS_ANALOG_DIRECT_OUTPUT_3 = "ANALOG_DIRECT_OUTPUT_3"
	CS_ANALOG_FILTER_3 = "ANALOG_FILTER_3"
	CS_ANALOG_METHOD_3 = "ANALOG_METHOD_3"
	CS_ANALOG_AMPLITUDE_3 = "ANALOG_AMPLITUDE_3"
	CS_ANALOG_OFFSET_3 = "ANALOG_OFFSET_3"
	CS_ANALOG_HIGH_3 = "ANALOG_HIGH_3"
	CS_ANALOG_LOW_3 = "ANALOG_LOW_3"
	CS_MARKER1_SKEW_3 = "MARKER1_SKEW_3"
	CS_MARKER1_METHOD_3 = "MARKER1_METHOD_3"
	CS_MARKER1_AMPLITUDE_3 = "MARKER1_AMPLITUDE_3"
	CS_MARKER1_OFFSET_3 = "MARKER1_OFFSET_3"
	CS_MARKER1_HIGH_3 = "MARKER1_HIGH_3"
	CS_MARKER1_LOW_3 = "MARKER1_LOW_3"
	CS_MARKER2_SKEW_3 = "MARKER2_SKEW_3"
	CS_MARKER2_METHOD_3 = "MARKER2_METHOD_3"
	CS_MARKER2_AMPLITUDE_3 = "MARKER2_AMPLITUDE_3"
	CS_MARKER2_OFFSET_3 = "MARKER2_OFFSET_3"
	CS_MARKER2_HIGH_3 = "MARKER2_HIGH_3"
	CS_MARKER2_LOW_3 = "MARKER2_LOW_3"
	CS_DIGITAL_METHOD_3 = "DIGITAL_METHOD_3"
	CS_DIGITAL_AMPLITUDE_3 = "DIGITAL_AMPLITUDE_3"
	CS_DIGITAL_OFFSET_3 = "DIGITAL_OFFSET_3"
	CS_DIGITAL_HIGH_3 = "DIGITAL_HIGH_3"
	CS_DIGITAL_LOW_3 = "DIGITAL_LOW_3"
	CS_EXTERNAL_ADD_3 = "EXTERNAL_ADD_3"
	CS_PHASE_DELAY_INPUT_METHOD_3 = "PHASE_DELAY_INPUT_METHOD_3"
	CS_PHASE_3 = "PHASE_3"
	CS_DELAY_IN_TIME_3 = "DELAY_IN_TIME_3"
	CS_DELAY_IN_POINTS_3 = "DELAY_IN_POINTS_3"
	CS_CHANNEL_SKEW_3 = "CHANNEL_SKEW_3"
	CS_DAC_RESOLUTION_4 = "DAC_RESOLUTION_4"
	CS_OUTPUT_WAVEFORM_NAME_4 = "OUTPUT_WAVEFORM_NAME_4"
	CS_CHANNEL_STATE_4 = "CHANNEL_STATE_4"
	CS_ANALOG_DIRECT_OUTPUT_4 = "ANALOG_DIRECT_OUTPUT_4"
	CS_ANALOG_FILTER_4 = "ANALOG_FILTER_4"
	CS_ANALOG_METHOD_4 = "ANALOG_METHOD_4"
	CS_ANALOG_AMPLITUDE_4 = "ANALOG_AMPLITUDE_4"
	CS_ANALOG_OFFSET_4 = "ANALOG_OFFSET_4"
	CS_ANALOG_HIGH_4 = "ANALOG_HIGH_4"
	CS_ANALOG_LOW_4 = "ANALOG_LOW_4"
	CS_MARKER1_SKEW_4 = "MARKER1_SKEW_4"
	CS_MARKER1_METHOD_4 = "MARKER1_METHOD_4"
	CS_MARKER1_AMPLITUDE_4 = "MARKER1_AMPLITUDE_4"
	CS_MARKER1_OFFSET_4 = "MARKER1_OFFSET_4"
	CS_MARKER1_HIGH_4 = "MARKER1_HIGH_4"
	CS_MARKER1_LOW_4 = "MARKER1_LOW_4"
	CS_MARKER2_SKEW_4 = "MARKER2_SKEW_4"
	CS_MARKER2_METHOD_4 = "MARKER2_METHOD_4"
	CS_MARKER2_AMPLITUDE_4 = "MARKER2_AMPLITUDE_4"
	CS_MARKER2_OFFSET_4 = "MARKER2_OFFSET_4"
	CS_MARKER2_HIGH_4 = "MARKER2_HIGH_4"
	CS_MARKER2_LOW_4 = "MARKER2_LOW_4"
	CS_DIGITAL_METHOD_4 = "DIGITAL_METHOD_4"
	CS_DIGITAL_AMPLITUDE_4 = "DIGITAL_AMPLITUDE_4"
	CS_DIGITAL_OFFSET_4 = "DIGITAL_OFFSET_4"
	CS_DIGITAL_HIGH_4 = "DIGITAL_HIGH_4"
	CS_DIGITAL_LOW_4 = "DIGITAL_LOW_4"
	CS_EXTERNAL_ADD_4 = "EXTERNAL_ADD_4"
	CS_PHASE_DELAY_INPUT_METHOD_4 = "PHASE_DELAY_INPUT_METHOD_4"
	CS_PHASE_4 = "PHASE_4"
	CS_DELAY_IN_TIME_4 = "DELAY_IN_TIME_4"
	CS_DELAY_IN_POINTS_4 = "DELAY_IN_POINTS_4"
	CS_CHANNEL_SKEW_4 = "CHANNEL_SKEW_4"
	CS_DAC_RESOLUTION_N = "DAC_RESOLUTION_"
	CS_OUTPUT_WAVEFORM_NAME_N = "OUTPUT_WAVEFORM_NAME_"
	CS_CHANNEL_STATE_N = "CHANNEL_STATE_"
	CS_ANALOG_DIRECT_OUTPUT_N = "ANALOG_DIRECT_OUTPUT_"
	CS_ANALOG_FILTER_N = "ANALOG_FILTER_"
	CS_ANALOG_METHOD_N = "ANALOG_METHOD_"
	CS_ANALOG_AMPLITUDE_N = "ANALOG_AMPLITUDE_"
	CS_ANALOG_OFFSET_N = "ANALOG_OFFSET_"
	CS_ANALOG_HIGH_N = "ANALOG_HIGH_"
	CS_ANALOG_LOW_N = "ANALOG_LOW_"
	CS_MARKER1_SKEW_N = "MARKER1_SKEW_"
	CS_MARKER1_METHOD_N = "MARKER1_METHOD_"
	CS_MARKER1_AMPLITUDE_N = "MARKER1_AMPLITUDE_"
	CS_MARKER1_OFFSET_N = "MARKER1_OFFSET_"
	CS_MARKER1_HIGH_N = "MARKER1_HIGH_"
	CS_MARKER1_LOW_N = "MARKER1_LOW_"
	CS_MARKER2_SKEW_N = "MARKER2_SKEW_"
	CS_MARKER2_METHOD_N = "MARKER2_METHOD_"
	CS_MARKER2_AMPLITUDE_N = "MARKER2_AMPLITUDE_"
	CS_MARKER2_OFFSET_N = "MARKER2_OFFSET_"
	CS_MARKER2_HIGH_N = "MARKER2_HIGH_"
	CS_MARKER2_LOW_N = "MARKER2_LOW_"
	CS_DIGITAL_METHOD_N = "DIGITAL_METHOD_"
	CS_DIGITAL_AMPLITUDE_N = "DIGITAL_AMPLITUDE_"
	CS_DIGITAL_OFFSET_N = "DIGITAL_OFFSET_"
	CS_DIGITAL_HIGH_N = "DIGITAL_HIGH_"
	CS_DIGITAL_LOW_N = "DIGITAL_LOW_"
	CS_EXTERNAL_ADD_N = "EXTERNAL_ADD_"
	CS_PHASE_DELAY_INPUT_METHOD_N = "PHASE_DELAY_INPUT_METHOD_"
	CS_PHASE_N = "PHASE_"
	CS_DELAY_IN_TIME_N = "DELAY_IN_TIME_"
	CS_DELAY_IN_POINTS_N = "DELAY_IN_POINTS_"
	CS_CHANNEL_SKEW_N = "CHANNEL_SKEW_"
	CS_DC_OUTPUT_LEVEL_1 = "DC_OUTPUT_LEVEL_1"
	CS_DC_OUTPUT_LEVEL_2 = "DC_OUTPUT_LEVEL_2"
	CS_DC_OUTPUT_LEVEL_3 = "DC_OUTPUT_LEVEL_3"
	CS_DC_OUTPUT_LEVEL_4 = "DC_OUTPUT_LEVEL_4"
	CS_DC_OUTPUT_LEVEL_N = "DC_OUTPUT_LEVEL_"
	CS_WAVEFORM_NAME_N = "WAVEFORM_NAME_"
	CS_WAVEFORM_TYPE_N = "WAVEFORM_TYPE_"
	CS_WAVEFORM_LENGTH_N = "WAVEFORM_LENGTH_"
	CS_WAVEFORM_TIMESTAMP_N = "WAVEFORM_TIMESTAMP_"
	CS_WAVEFORM_DATA_N = "WAVEFORM_DATA_"
	CS_SEQUENCE_WAIT_N = "SEQUENCE_WAIT_"
	CS_SEQUENCE_LOOP_N = "SEQUENCE_LOOP_"
	CS_SEQUENCE_JUMP_N = "SEQUENCE_JUMP_"
	CS_SEQUENCE_GOTO_N = "SEQUENCE_GOTO_"
	CS_SEQUENCE_WAVEFORM_NAME_CH_N_N = "SEQUENCE_WAVEFORM_NAME_CH_"
	CS_SEQUENCE_IS_SUBSEQUENCE_N = "SEQUENCE_IS_SUBSEQ_"
	CS_SEQUENCE_SUBSEQUENCE_NAME_N = "SEQUENCE_SUBSEQ_NAME_"
	CS_SUBSEQUENCE_NAME_O = "SUBSEQ_NAME_"
	CS_SUBSEQUENCE_TIMESTAMP_O = "SUBSEQ_TIMESTAMP_"
	CS_SUBSEQUENCE_LENGTH_O = "SUBSEQ_LENGTH_"
	CS_SUBSEQUENCE_LOOP_E_O_U = "SUBSEQ_LOOP_"
	CS_SUBSEQUENCE_WAVEFORM_NAME_CH_X_E_O_U = "SUBSEQ_WAVEFORM_NAME_CH_"
SEQUENCE_JUMP_TARGET_TYPE_OFF = 0
SEQUENCE_JUMP_TARGET_TYPE_NEXT = -1



# -*- coding: utf-8 -*-
"""
    lantz.drivers.legacy.coherent
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :company: Coherent Inc.
    :description: Lasers and Lasers Systems.
    :website: http://www.coherent.com/

    ----

    :copyright: 2015 by Lantz Authors, see AUTHORS for more details.
    :license: BSD, see LICENSE for more details.
"""

from .innova import Innova300C, ArgonInnova300C, KryptonInnova300C

__all__ = ['Innova300C', 'ArgonInnova300C', 'KryptonInnova300C']
